/*
 * z_tray.c: Trayicon functions
 *
 * Copyright (C) 2014  Matthias Zartmann.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 * USA
 *
 * Authors:
 *     Matthias Zartmann <matthias.zartmann@gmx.de>
 * Created on: 4 Apr 2014
 */

#include "zr_tray.h"
#include "zr_app.h"
#include "z_util.h"


#include <stdlib.h>

static void on_mitem_click(GtkMenuItem *menuitem, zr_tray * p_zr_tray);
static void on_radioitem_click(GtkMenuItem *menuitem, zr_cfg_station * p_station);
static void tray_popup_menu(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer user_data);
static gboolean tray_scroll(GtkStatusIcon *status_icon, GdkEvent *event, gpointer user_data);
static void tray_activate(GtkStatusIcon *status_icon, zr_tray * p_zr_tray);
static GtkImageMenuItem * createStockMenuItem(GtkMenu * p_menu, const gchar * p_label, const gchar * p_stock_is, const gchar * name);
static GtkMenuItem * createSeperatorItem(GtkMenu * p_menu);
static GtkMenuItem * createRadioItem(GtkMenu * p_menu, zr_cfg_station * station);

/**
 * Create the Tray
 * @param pconfig config instance
 * @return the Tray
 */
zr_tray * zr_tray_create(zr_cfg * pconfig)
{
	zr_tray * v_result;
	GError * error = NULL;
	g_debug("TRAY Create");

	v_result = g_new(struct zr_tray, 1);

	v_result->tray_menu = GTK_MENU(gtk_menu_new());
	gtk_widget_show(GTK_WIDGET(v_result->tray_menu));

	v_result->play = createStockMenuItem(v_result->tray_menu, "Play", GTK_STOCK_MEDIA_PLAY, ZR_NAME_MITEM_PLAY);
	v_result->pause = createStockMenuItem(v_result->tray_menu, "Pause", GTK_STOCK_MEDIA_PAUSE, ZR_NAME_MITEM_PAUSE);
	v_result->reset = createStockMenuItem(v_result->tray_menu, "Reset", GTK_STOCK_REFRESH, ZR_NAME_MITEM_RESET);
	v_result->exit = createStockMenuItem(v_result->tray_menu, "Exit", GTK_STOCK_QUIT, ZR_NAME_MITEM_EXIT);
	createSeperatorItem(v_result->tray_menu);

	int i = 0;
	zr_cfg_station * station = &g_array_index(pconfig->stations, zr_cfg_station, i);
	while (station->key != NULL)
	{

		GtkMenuItem * item = createRadioItem(v_result->tray_menu, station);
		g_signal_connect(item, "activate", G_CALLBACK (on_radioitem_click), station);
		i++;
		station = &g_array_index(pconfig->stations, zr_cfg_station, i);
	}

	//Volume Slider

	g_signal_connect(v_result->pause, "activate", G_CALLBACK (on_mitem_click), v_result);
	g_signal_connect(v_result->play, "activate", G_CALLBACK (on_mitem_click), v_result);
	g_signal_connect(v_result->reset, "activate", G_CALLBACK (on_mitem_click), v_result);
	g_signal_connect(v_result->exit, "activate", G_CALLBACK (on_mitem_click), v_result);


	GtkIconTheme * icontheme = gtk_icon_theme_new ();
	GdkPixbuf * pb_tray_icon;
	error = NULL;
	pb_tray_icon = gtk_icon_theme_load_icon (icontheme, "zradio",32,GTK_ICON_LOOKUP_NO_SVG,&error);

	g_object_unref(G_OBJECT(icontheme));
	if (pb_tray_icon == NULL)
	{
		g_error("Can't load resource tray_icon.png. %s", error->message);
	}
	v_result->tray_icon = gtk_status_icon_new_from_pixbuf(pb_tray_icon);
	g_object_unref(pb_tray_icon);

	g_debug("v_result->tray_icon=%p", v_result->tray_icon);
	gtk_status_icon_set_tooltip_text(v_result->tray_icon, "ZRadio Tray");

	g_signal_connect(GTK_STATUS_ICON (v_result->tray_icon), "activate", G_CALLBACK (tray_activate), v_result);
	g_signal_connect(GTK_STATUS_ICON(v_result->tray_icon), "popup-menu", G_CALLBACK(tray_popup_menu), v_result);

	g_signal_connect(GTK_STATUS_ICON(v_result->tray_icon), "scroll-event", G_CALLBACK(tray_scroll), v_result);

	gtk_status_icon_set_visible(v_result->tray_icon, TRUE);

	return v_result;
}

/**
 * Free The Tray
 * @param p_tray
 */
void zr_tray_free(zr_tray * p_tray)
{

	gtk_widget_destroy(GTK_WIDGET(p_tray->tray_menu));

	g_object_unref(G_OBJECT(p_tray->tray_icon));
	g_free(p_tray);
}

/**
 * Show the Tray
 * @param p_tray
 */
void zr_tray_show(zr_tray * p_tray)
{

	g_debug("show Tray");
	gtk_status_icon_set_visible(p_tray->tray_icon, TRUE);
}

/**
 * Set the new Tooltip
 * @param p_tray
 * @param p_Tooltip tooltip string
 */
void zr_tray_set_tooltip(zr_tray * p_tray, gchar* p_Tooltip)
{
	gtk_status_icon_set_tooltip_text(p_tray->tray_icon, p_Tooltip);
}

/*****************************************************************************************************************
 internal functions
 *****************************************************************************************************************/
static GtkImageMenuItem * createStockMenuItem(GtkMenu * p_menu, const gchar * p_label, const gchar * p_stock_is, const gchar * name)
{
	GtkImageMenuItem * item;

	item = GTK_IMAGE_MENU_ITEM(gtk_image_menu_item_new_from_stock(p_stock_is,NULL));
	if (item == NULL)
	{
		perror("Error Create Menu");
	}
	gtk_widget_set_name(GTK_WIDGET(item), name);
	gtk_menu_item_set_label(GTK_MENU_ITEM(item), p_label);
	gtk_menu_shell_append(GTK_MENU_SHELL(p_menu), GTK_WIDGET(item));
	gtk_widget_show(GTK_WIDGET(item));

	return item;
}

static GtkMenuItem * createRadioItem(GtkMenu * p_menu, zr_cfg_station * station)
{
	GtkMenuItem * item;

	item = GTK_MENU_ITEM(gtk_menu_item_new_with_label(station->name));
	if (item == NULL)
	{
		perror("Error Create Menuitem from");
	}
	gtk_widget_set_name(GTK_WIDGET(item), station->key);
	gtk_menu_shell_append(GTK_MENU_SHELL(p_menu), GTK_WIDGET(item));
	gtk_widget_show(GTK_WIDGET(item));

	return item;
}

static GtkMenuItem * createSeperatorItem(GtkMenu * p_menu)
{
	GtkMenuItem * item;

	item = GTK_MENU_ITEM(gtk_separator_menu_item_new());
	if (item == NULL)
	{
		perror("Error Create separator- Menu");
	}
	gtk_menu_shell_append(GTK_MENU_SHELL(p_menu), GTK_WIDGET(item));
	gtk_widget_show(GTK_WIDGET(item));

	return item;
}

static void tray_popup_menu(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer user_data)
{
	struct zr_tray * p_tray = (struct zr_tray *) user_data;
	gtk_menu_popup(p_tray->tray_menu,
	NULL,
	NULL, gtk_status_icon_position_menu, status_icon, button, activate_time);

}

static gboolean tray_scroll(GtkStatusIcon *status_icon, GdkEvent *event, gpointer user_data)
{

	if (event->scroll.delta_y == 1.0 || event->scroll.delta_x == 1.0)
		zr_app_emit_event(ZR_APP_VOL_CHANGE, "UP");
	else if (event->scroll.delta_y == -1.0 || event->scroll.delta_x == -1.0)
		zr_app_emit_event(ZR_APP_VOL_CHANGE, "DOWN");

	return TRUE;
}

static void on_mitem_click(GtkMenuItem *menuitem, zr_tray * p_zr_tray)
{

	const gchar * name = gtk_widget_get_name(GTK_WIDGET(menuitem));

	if (g_strcmp0(name, ZR_NAME_MITEM_PLAY) == 0)
	{

		zr_app_emit_event(ZR_APP_MSG_PLAY, "LAST");
		g_message("Play");
	} else if (g_strcmp0(name, ZR_NAME_MITEM_PAUSE) == 0)
	{
		zr_app_emit_event(ZR_APP_MSG_PAUSE, NULL);
		g_message("Pause");
	} else if (g_strcmp0(name, ZR_NAME_MITEM_RESET) == 0)
	{
		zr_app_emit_event(ZR_APP_MSG_RESET, NULL);
		g_message("Reset");
	} else if (g_strcmp0(name, ZR_NAME_MITEM_EXIT) == 0)
	{
		zr_app_emit_event(ZR_APP_MSG_CLOSE, NULL);
		g_message("CLOSE APP");
	}
}

static void on_radioitem_click(GtkMenuItem *menuitem, zr_cfg_station * station)
{
	zr_app_emit_event(ZR_APP_MSG_PLAY, station->url);
}

static void tray_activate(GtkStatusIcon *status_icon, zr_tray * p_zr_tray)
{
	zr_app_emit_event(ZR_APP_MSG_PAUSE, NULL);
}

